// Forlix PlaySound
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2009-2013 Dominik Friedrichs


Playback_PluginStart()
{
  CreateTimer(REGENERATION_INTERVAL,
              Timer_SoundRegenerate,
              INVALID_HANDLE, TIMER_REPEAT);
  return;
}


bool:PlaySound(client,
               const String:snd_name[],
               bool:r_notfound=false)
{
  // client can be zero for server console
  decl String:snd_path[PLATFORM_MAX_PATH];

  static String:snd_path_last[PLATFORM_MAX_PATH];
  static snd_chan = PLAYBACK_CHANNEL_BASE;

  if(!GetTrieString(h_trie_snds, snd_name, snd_path, sizeof(snd_path)))
  {
    if(r_notfound)
      ReplyWithDeadTimeBlock(client, "%t", "REQUESTED_SOUND_NOT_FOUND");

    return(false);
  }

  if(client
  && !successive_sounds)
  // server console can play sounds no matter what
    ReplyWithDeadTimeBlock(client, "%t", "SOUND_PLAYBACK_DISABLED");
  else
  if(client
  && GetClientListeningFlags(client) & VOICE_MUTED)
  // player muted
    ReplyWithDeadTimeBlock(client, "%t", "SOUND_PLAYBACK_MUTED");
  else
  if(!client
  || p_sounds[client] >= 1)
  // console plays or player plays and has enough credits for at least 1 sound
  {
    DeadTimeBlock(client);

    new playnum = 0;
    decl playto[MAXPLAYERS];

    for(new i = 1; i <= MaxClients; i++)
    if(IsClientInGame(i)
    && !(p_flags[i] & PLAYER_SOUNDS_MUTED)
    && (!client || !IsClientMuted(i, client)))
    // build list of clients to actually play to
      playto[playnum++] = i;

    if(playnum)
    {
      if(!sound_overlapping
      && snd_path_last[0])
      // no overlapping, stop the previous sound first
        StopSoundForClients(playto, playnum, snd_path_last, snd_chan);

      // alternate between two channels
      // avoids bug where sounds may be choked off
      snd_chan ^= (1<<0);

      EmitLocalSoundForClients(playto, playnum, snd_path, snd_chan);
      strcopy(snd_path_last, sizeof(snd_path_last), snd_path);
    }
    else
      snd_path_last[0] = '\0';

    if(client)
    // player plays sound
    {
      decl String:name[MAX_NAME_LEN];
      name[0] = '\0';

      GetClientName(client, name, sizeof(name));

      PrintToChatAll("%t", "PLAYER_PLAYED_SOUND", name, snd_name);
      p_sounds[client] -= 1;
    }
    else
    // console plays sound
      PrintToChatAll("%t", "CONSOLE_PLAYED_SOUND", snd_name);

    if(log_playbacks)
    {
      new userid = 0;
      decl String:steamid[32];
      decl String:teamname[32];

      if(client)
      {
        userid = GetClientUserId(client);

        if(!GetClientAuthString(client, steamid, sizeof(steamid)))
          strcopy(steamid, sizeof(steamid), "STEAM_ID_PENDING");

        GetTeamName(GetClientTeam(client), teamname, sizeof(teamname));
      }
      else
      {
        steamid = "Console";
        teamname = "Console";
      }

      LogToGame(LOG_MSG_PLAYBACK, client, userid, steamid, teamname, snd_name);
    }
  }
  else
    ReplyWithDeadTimeBlock(client, "%t", "WAIT_FOR_REGENERATE");

  return(true);
}


EmitLocalSoundForClients(const clients[],
                         numClients,
                         const String:sample[],
                         channel)
{
  for(new i = 0; i < numClients; i++)
  {
    decl client[1];
    client[0] = clients[i];

    EmitSound(client, 1, sample, client[0], channel);
  }

  return;
}


StopSoundForClients(const clients[],
                    numClients,
                    const String:sample[],
                    channel)
{
  for(new i = 0; i < numClients; i++)
    StopSound(clients[i], channel, sample);

  return;
}


public Action:Timer_SoundRegenerate(Handle:timer)
{
  if(!regenerate_per_minute
  || !successive_sounds)
    return(Plugin_Continue);

  new Float:regen = regenerate_per_minute*(REGENERATION_INTERVAL/60.0);

  for(new client = 1; client <= MaxClients; client++)
  if(IsClientInGame(client)
  && p_sounds[client] < successive_sounds)
    p_sounds[client] += regen;

  return(Plugin_Continue);
}


FullRegenerate()
{
  for(new client = 1; client <= MaxClients; client++)
  if(IsClientInGame(client))
    p_sounds[client] = float(successive_sounds);

  return;
}
